package com.huimv.eartag2.eartag.listener;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.huimv.eartag2.common.mq.Const;
import com.huimv.eartag2.common.utils.DateUtil;
import com.huimv.eartag2.common.utils.MathUtil;
import com.huimv.eartag2.eartag.service.IDeviceService;
import com.huimv.eartag2.eartag.service.IEartagService;
import lombok.extern.slf4j.Slf4j;
//import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Project : huimv.shiwan
 * @Package : com.huimv.biosafety.uface.controller
 * @Description : TODO
 * @Version : 1.0
 * @Author : ZhuoNing
 * @Create : 2020-12-25
 **/
@Component
@Slf4j
public class EartagListener {
    @Autowired
    private IDeviceService deviceService;
    @Autowired
    private IEartagService eartagService;
    @Autowired
    private RabbitTemplate rabbitTemplate;

    private static final String TOPIC = "test-topic";

    /**
     * @Method : handleAskText
     * @Description : 处理请求
     * @Params : [askText]
     * @Return : java.util.Map
     * @Author : ZhuoNing
     * @Date : 2022/2/9
     * @Time : 13:38
     */
    private Map handleAskText(String askText) {
        Map outMap = new HashMap();
        JSONObject dataJo = new JSONObject();
        String type = "";
        String[] dataArray = askText.split("\\+");
        String cmdHeader = dataArray[0];
        if (cmdHeader.trim().equalsIgnoreCase("hm")) {
            //采集器原始数据
            String device = dataArray[1];
            String cmd = dataArray[2];
            if (cmd.trim().equalsIgnoreCase("0")) {
                //不需要处理
            } else if (cmd.trim().equalsIgnoreCase("1")) {
                //获取设备编码
            } else if (cmd.trim().equalsIgnoreCase("2")) {
                //获取服务器时间命令
            } else if (cmd.trim().equalsIgnoreCase("3")) {
                //心跳包命令
                dataJo.put("device", device);
                type = "device-heartbeat";
            } else if (cmd.trim().equalsIgnoreCase("4")) {
                //环境温度-设备、数值、上传时间
                dataJo.put("device", dataArray[1]);
                dataJo.put("temp", dataArray[4]);
                dataJo.put("askTime", dataArray[5]);
                type = "device-temp";
            } else {
                log.error("##当前数据为非法数据>>" + askText);
            }
        } else if (cmdHeader.trim().equalsIgnoreCase("zj")) {
//            String device = dataArray[1];
            //-- 耳标数据 --//
            //计算耳标温度
            Double earTemp = 0D;
            Double earTemp2 = 0D;
            String other = dataArray[9];
            if (other != null && other.trim().length() > 0) {
                String[] otherArray = other.split("#");
                String resi = otherArray[2];
                if (otherArray.length>3){
                    String resi2 = otherArray[3];
                    if (null != resi2 && !"0".equals(resi2)){
                        //耳面温度
                        Double temp2 = MathUtil.ln( Integer.parseInt(resi2));
                        earTemp2 = new BigDecimal(temp2).setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                    }
                }

                int resiInt = Integer.parseInt(resi);
                if (resiInt != 0) {
                    Double temp = MathUtil.ln(resiInt);
                    earTemp = new BigDecimal(temp).setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue();
                }

            }
            //计算运动量//
            Integer act1Int = eartagService.countAct(dataArray[2], dataArray[6]);
            //命令头
            dataJo.put("cmdHeader", dataArray[0]);
            //采集器id
            dataJo.put("device", dataArray[1]);
            //耳标号
            dataJo.put("earmark", dataArray[2]);
            //电池电量
            dataJo.put("bat", dataArray[3]);
            //耳标温度
            dataJo.put("earTemp", dataArray[4]);
            dataJo.put("earTemp1", earTemp);
            dataJo.put("earTemp2", earTemp2);
            //环境温度
            dataJo.put("envTemp", dataArray[5]);
            dataJo.put("envTemp1", MathUtil.countEnvtemp(dataArray[5], 2));
            //运动量
            dataJo.put("act", dataArray[6]);
            dataJo.put("act1", act1Int);
            //信号强度
            dataJo.put("signal", dataArray[7]);
            //采集时间
            dataJo.put("askTime", dataArray[8]);
            //预留字段
            dataJo.put("other", dataArray[9]);
            type = "eartag";
        }
        outMap.put("type", type);
        outMap.put("data", dataJo);
        return outMap;
    }

    /**
     * @Method : processRawdata
     * @Description :
     * @Params : [RawMap]
     * @Return : void
     * @Author : ZhuoNing
     * @Date : 2022/3/12
     * @Time : 17:53
     */
//    @RabbitListener(queues = Const.QUEUE_ASK_EARTAG )
    @RabbitListener(queues = "#{'direct.askEartag.queue' + ${rabbitmq.queues}}")
    @RabbitHandler
    public void processRawdata(Map RawMap) throws ParseException, IOException {
         log.info("开始处理原始耳标数据------>"+RawMap);
        String askText = RawMap.get("askText").toString();

        //{处理请求报文}
        Map askMap = handleAskText(askText);
        if (askMap == null){
            return;
        }
        String type = askMap.get("type").toString();
        JSONObject dataJo = (JSONObject) askMap.get("data");
        if (type.trim().equalsIgnoreCase("eartag")) {
            String deviceCode = dataJo.getString("device");
            String farmId = deviceService.getFarmIdByDeviceCode(deviceCode);
            System.out.println("####### farmId=" + farmId);
            if (ObjectUtil.isEmpty(farmId)){
                log.error("出错:该请求的FarmId为空.请检查该设备编码[" + deviceCode + "]是否存在FarmId.");
                return;
            }
            dataJo.put("farmId",farmId);
            //计算小时运动量
            rabbitTemplate.convertAndSend(Const.EXCHANGE_COUNT_ACT_HOUR,Const.ROUTING_KEY_COUNT_ACT_HOUR,dataJo);

            //更新设备注册信息
            rabbitTemplate.convertAndSend(Const.EXCHANGE_DEVICE_REGIST,Const.ROUTING_KEY_DEVICE_REGIST,dataJo);

            //更新耳标注册信息
            rabbitTemplate.convertAndSend(Const.EXCHANGE_EARTAG_REGIST,Const.ROUTING_KEY_EARTAG_REGIST,dataJo);

            //更新设备在线统计
            rabbitTemplate.convertAndSend(Const.EXCHANGE_DEVICE_COUNT,Const.ROUTING_KEY_DEVICE_COUNT,dataJo);

            //更新耳标在线统计
            rabbitTemplate.convertAndSend(Const.EXCHANGE_EARTAG_COUNT,Const.ROUTING_KEY_EARTAG_COUNT,dataJo);

            //更新耳标在线状态
            rabbitTemplate.convertAndSend(Const.EXCHANGE_EARTAG_LIVE,Const.ROUTING_KEY_EARTAG_LIVE,dataJo);

            //更新耳标设备在线关联统计
            rabbitTemplate.convertAndSend(Const.EXCHANGE_EARTAG_DEVICE_ONLINE,Const.ROUTING_KEY_EARTAG_DEVICE_ONLINE,dataJo);

            //异常统计和异常报警
            rabbitTemplate.convertAndSend(Const.EXCHANGE_ABNORMAL,Const.ROUTING_KEY_ABNORMAL,dataJo);

            //判断该设备编号是否存在牧场缓存在线集合当中
            rabbitTemplate.convertAndSend(Const.EXCHANGE_AllSTATUS,Const.ROUTING_KEY_AllSTATUS,dataJo);


        }
        eartagService.handleEartag(dataJo);
    }



    @RabbitListener(queues = Const.QUEUE_COUNT_ACT_HOUR)
    @RabbitHandler
    public void countActHour(JSONObject dataJo) throws ParseException, IOException {
        log.info("开始计算小时运动量------>"+dataJo);

    }










    public boolean getAskTime(String askTime)   {
        String newAskTime = askTime.substring(0, 4) + "-" + askTime.substring(4, 6) + "-" + askTime.substring(6, 8) + " " + askTime.substring(8, 10) + ":" + askTime.substring(10, 12) + ":" + askTime.substring(12, 14);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            Date targetDate = sdf.parse(newAskTime);
            Date currentDate = new Date();

            // 获取时间戳
            long targetTimestamp = targetDate.getTime();
            long currentTimestamp = currentDate.getTime();

            // 设置时间差阈值，例如10分钟
            long maxTimeDifference = 60 * 1000; // 10 minutes in milliseconds

            // 计算时间差
            long timeDifference = Math.abs(targetTimestamp - currentTimestamp);

            if (timeDifference <= maxTimeDifference) {
                return true;
            } else {
                return false;
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        return false;
    }

    public void writeTxt(String conent,String file) throws ParseException, IOException {
        String todayText = getTodayDateText();
        File f = new File("/opt/log/log_"+file+"_"+todayText+".txt");
        if (f.exists()) {
        } else {
            f.createNewFile();// 不存在则创建
        }
        BufferedWriter output = new BufferedWriter(new FileWriter(f,true));//true,则追加写入text文本
        output.write(conent);
        output.write("\r\n");//换行
        output.flush();
        output.close();
    }

    public String getTodayDateText() throws ParseException {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        return sdf.format(new Date());
    }

    private boolean checkTestRange(String earmark) {
        Long c = Long.parseLong(earmark);
        Long a = 122083123610001L;
        Long b = 122083123610100L;
        return (c >= a && c < b) || (c > a && c <= b);
    }
}
